@@ -1,33 +1,37 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::fmt;
-use std::marker::PhantomData;
-use std::ops;
-
 #[cfg(any(feature = "v1_20", feature = "dox"))]
 #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
 use std::ptr;
-
-use crate::Buffer;
-use crate::BufferRef;
-use crate::Caps;
-use crate::CapsRef;
-use crate::ClockTime;
+use std::{fmt, marker::PhantomData, ops};
 
 use glib::translate::*;
 
+use crate::{Buffer, BufferRef, Caps, CapsRef, ClockTime};
+
 pub unsafe trait MetaAPI: Sync + Send + Sized {
     type GstType;
 
     #[doc(alias = "get_meta_api")]
     fn meta_api() -> glib::Type;
+}
 
+pub trait MetaAPIExt: MetaAPI {
+    unsafe fn from_ptr(buffer: &BufferRef, ptr: *const Self::GstType) -> MetaRef<Self>;
+    unsafe fn from_mut_ptr<T>(
+        buffer: &mut BufferRef,
+        ptr: *mut Self::GstType,
+    ) -> MetaRefMut<Self, T>;
+}
+
+impl<A: MetaAPI> MetaAPIExt for A {
+    #[inline]
     unsafe fn from_ptr(buffer: &BufferRef, ptr: *const Self::GstType) -> MetaRef<Self> {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         let meta_api = Self::meta_api();
         if meta_api != glib::Type::INVALID {
-            assert_eq!(
+            debug_assert_eq!(
                 meta_api,
                 from_glib((*(*(ptr as *const ffi::GstMeta)).info).api)
             )
@@ -39,15 +43,16 @@ pub unsafe trait MetaAPI: Sync + Send + Sized {
         }
     }
 
+    #[inline]
     unsafe fn from_mut_ptr<T>(
         buffer: &mut BufferRef,
         ptr: *mut Self::GstType,
     ) -> MetaRefMut<Self, T> {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         let meta_api = Self::meta_api();
         if meta_api != glib::Type::INVALID {
-            assert_eq!(
+            debug_assert_eq!(
                 meta_api,
                 from_glib((*(*(ptr as *const ffi::GstMeta)).info).api)
             )
@@ -102,12 +107,14 @@ impl<'a, T: fmt::Debug + 'a, U> fmt::Debug for MetaRefMut<'a, T, U> {
 impl<'a, T> ops::Deref for MetaRef<'a, T> {
     type Target = T;
 
+    #[inline]
     fn deref(&self) -> &T {
         self.meta
     }
 }
 
 impl<'a, T> AsRef<MetaRef<'a, T>> for MetaRef<'a, T> {
+    #[inline]
     fn as_ref(&self) -> &MetaRef<'a, T> {
         self
     }
@@ -116,18 +123,21 @@ impl<'a, T> AsRef<MetaRef<'a, T>> for MetaRef<'a, T> {
 impl<'a, T, U> ops::Deref for MetaRefMut<'a, T, U> {
     type Target = T;
 
+    #[inline]
     fn deref(&self) -> &T {
         self.meta
     }
 }
 
 impl<'a, T, U> ops::DerefMut for MetaRefMut<'a, T, U> {
+    #[inline]
     fn deref_mut(&mut self) -> &mut T {
         self.meta
     }
 }
 
 impl<'a, T, U> AsRef<MetaRef<'a, T>> for MetaRefMut<'a, T, U> {
+    #[inline]
     fn as_ref(&self) -> &MetaRef<'a, T> {
         unsafe { &*(self as *const MetaRefMut<'a, T, U> as *const MetaRef<'a, T>) }
     }
@@ -135,6 +145,7 @@ impl<'a, T, U> AsRef<MetaRef<'a, T>> for MetaRefMut<'a, T, U> {
 
 impl<'a, T> MetaRef<'a, T> {
     #[doc(alias = "get_api")]
+    #[inline]
     pub fn api(&self) -> glib::Type {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -143,6 +154,7 @@ impl<'a, T> MetaRef<'a, T> {
         }
     }
 
+    #[inline]
     pub fn flags(&self) -> crate::MetaFlags {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -150,6 +162,7 @@ impl<'a, T> MetaRef<'a, T> {
         }
     }
 
+    #[inline]
     pub fn type_(&self) -> glib::Type {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -162,6 +175,7 @@ impl<'a, T> MetaRef<'a, T> {
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
     #[doc(alias = "get_seqnum")]
     #[doc(alias = "gst_meta_get_seqnum")]
+    #[inline]
     pub fn seqnum(&self) -> MetaSeqnum {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -169,6 +183,7 @@ impl<'a, T> MetaRef<'a, T> {
         }
     }
 
+    #[inline]
     pub fn as_ptr(&self) -> *const T::GstType
     where
         T: MetaAPI,
@@ -178,6 +193,7 @@ impl<'a, T> MetaRef<'a, T> {
 }
 
 impl<'a> MetaRef<'a, Meta> {
+    #[inline]
     pub fn downcast_ref<T: MetaAPI>(&self) -> Option<&MetaRef<'a, T>> {
         let target_type = T::meta_api();
         let type_ = self.api();
@@ -191,6 +207,7 @@ impl<'a> MetaRef<'a, Meta> {
 
     #[cfg(any(feature = "v1_20", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[inline]
     pub fn try_as_custom_meta(&self) -> Option<&MetaRef<'a, CustomMeta>> {
         unsafe {
             if ffi::gst_meta_info_is_custom(&*self.0.info) == glib::ffi::GFALSE {
@@ -204,6 +221,7 @@ impl<'a> MetaRef<'a, Meta> {
 
 impl<'a, T, U> MetaRefMut<'a, T, U> {
     #[doc(alias = "get_api")]
+    #[inline]
     pub fn api(&self) -> glib::Type {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -212,6 +230,7 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
         }
     }
 
+    #[inline]
     pub fn flags(&self) -> crate::MetaFlags {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -219,6 +238,7 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
         }
     }
 
+    #[inline]
     pub fn type_(&self) -> glib::Type {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -231,6 +251,7 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
     #[doc(alias = "get_seqnum")]
     #[doc(alias = "gst_meta_get_seqnum")]
+    #[inline]
     pub fn seqnum(&self) -> u64 {
         unsafe {
             let meta = self.meta as *const _ as *const ffi::GstMeta;
@@ -238,6 +259,7 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
         }
     }
 
+    #[inline]
     pub fn as_ptr(&self) -> *const T::GstType
     where
         T: MetaAPI,
@@ -245,6 +267,7 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
         self.meta as *const _ as *const <T as MetaAPI>::GstType
     }
 
+    #[inline]
     pub fn as_mut_ptr(&mut self) -> *mut T::GstType
     where
         T: MetaAPI,
@@ -265,7 +288,7 @@ impl<'a, T> MetaRefMut<'a, T, Standalone> {
                 self.buffer.as_mut_ptr(),
                 self.meta as *mut T as *mut ffi::GstMeta,
             );
-            assert_ne!(res, glib::ffi::GFALSE);
+            debug_assert_ne!(res, glib::ffi::GFALSE);
 
             Ok(())
         }
@@ -273,6 +296,7 @@ impl<'a, T> MetaRefMut<'a, T, Standalone> {
 }
 
 impl<'a, U> MetaRefMut<'a, Meta, U> {
+    #[inline]
     pub fn downcast_ref<T: MetaAPI>(&mut self) -> Option<&MetaRefMut<'a, T, U>> {
         let target_type = T::meta_api();
         let type_ = self.api();
@@ -284,6 +308,7 @@ impl<'a, U> MetaRefMut<'a, Meta, U> {
         }
     }
 
+    #[inline]
     pub fn downcast_mut<T: MetaAPI>(&mut self) -> Option<&mut MetaRefMut<'a, T, U>> {
         let target_type = T::meta_api();
         let type_ = self.api();
@@ -299,6 +324,7 @@ impl<'a, U> MetaRefMut<'a, Meta, U> {
 
     #[cfg(any(feature = "v1_20", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[inline]
     pub fn try_as_custom_meta(&self) -> Option<&MetaRefMut<'a, CustomMeta, U>> {
         unsafe {
             if ffi::gst_meta_info_is_custom(&*self.0.info) == glib::ffi::GFALSE {
@@ -311,6 +337,7 @@ impl<'a, U> MetaRefMut<'a, Meta, U> {
 
     #[cfg(any(feature = "v1_20", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[inline]
     pub fn try_as_mut_custom_meta(&mut self) -> Option<&mut MetaRefMut<'a, CustomMeta, U>> {
         unsafe {
             if ffi::gst_meta_info_is_custom(&*self.0.info) == glib::ffi::GFALSE {
@@ -331,10 +358,12 @@ unsafe impl Sync for Meta {}
 
 impl Meta {
     #[doc(alias = "get_api")]
+    #[inline]
     fn api(&self) -> glib::Type {
         unsafe { glib::Type::from_glib((*self.0.info).api) }
     }
 
+    #[inline]
     pub fn flags(&self) -> crate::MetaFlags {
         unsafe { from_glib(self.0.flags) }
     }
@@ -343,6 +372,7 @@ impl Meta {
 unsafe impl MetaAPI for Meta {
     type GstType = ffi::GstMeta;
 
+    #[inline]
     fn meta_api() -> glib::Type {
         glib::Type::INVALID
     }
@@ -376,11 +406,13 @@ impl ParentBufferMeta {
     }
 
     #[doc(alias = "get_parent")]
+    #[inline]
     pub fn parent(&self) -> &BufferRef {
         unsafe { BufferRef::from_ptr(self.0.buffer) }
     }
 
     #[doc(alias = "get_parent_owned")]
+    #[inline]
     pub fn parent_owned(&self) -> Buffer {
         unsafe { from_glib_none(self.0.buffer) }
     }
@@ -390,6 +422,7 @@ unsafe impl MetaAPI for ParentBufferMeta {
     type GstType = ffi::GstParentBufferMeta;
 
     #[doc(alias = "gst_parent_buffer_meta_api_get_type")]
+    #[inline]
     fn meta_api() -> glib::Type {
         unsafe { from_glib(ffi::gst_parent_buffer_meta_api_get_type()) }
     }
@@ -423,11 +456,13 @@ impl ProtectionMeta {
     }
 
     #[doc(alias = "get_info")]
+    #[inline]
     pub fn info(&self) -> &crate::StructureRef {
         unsafe { crate::StructureRef::from_glib_borrow(self.0.info) }
     }
 
     #[doc(alias = "get_info_mut")]
+    #[inline]
     pub fn info_mut(&mut self) -> &mut crate::StructureRef {
         unsafe { crate::StructureRef::from_glib_borrow_mut(self.0.info) }
     }
@@ -437,6 +472,7 @@ unsafe impl MetaAPI for ProtectionMeta {
     type GstType = ffi::GstProtectionMeta;
 
     #[doc(alias = "gst_protection_meta_api_get_type")]
+    #[inline]
     fn meta_api() -> glib::Type {
         unsafe { from_glib(ffi::gst_protection_meta_api_get_type()) }
     }
@@ -479,21 +515,25 @@ impl ReferenceTimestampMeta {
     }
 
     #[doc(alias = "get_reference")]
+    #[inline]
     pub fn reference(&self) -> &CapsRef {
         unsafe { CapsRef::from_ptr(self.0.reference) }
     }
 
     #[doc(alias = "get_parent_owned")]
+    #[inline]
     pub fn parent_owned(&self) -> Caps {
         unsafe { from_glib_none(self.0.reference) }
     }
 
     #[doc(alias = "get_timestamp")]
+    #[inline]
     pub fn timestamp(&self) -> ClockTime {
         unsafe { try_from_glib(self.0.timestamp).expect("undefined timestamp") }
     }
 
     #[doc(alias = "get_duration")]
+    #[inline]
     pub fn duration(&self) -> Option<ClockTime> {
         unsafe { from_glib(self.0.duration) }
     }
@@ -503,6 +543,7 @@ unsafe impl MetaAPI for ReferenceTimestampMeta {
     type GstType = ffi::GstReferenceTimestampMeta;
 
     #[doc(alias = "gst_reference_timestamp_meta_api_get_type")]
+    #[inline]
     fn meta_api() -> glib::Type {
         unsafe { from_glib(ffi::gst_reference_timestamp_meta_api_get_type()) }
     }
@@ -660,6 +701,7 @@ impl CustomMeta {
     }
 
     #[doc(alias = "gst_custom_meta_get_structure")]
+    #[inline]
     pub fn structure(&self) -> &crate::StructureRef {
         unsafe {
             crate::StructureRef::from_glib_borrow(ffi::gst_custom_meta_get_structure(mut_override(
@@ -669,6 +711,7 @@ impl CustomMeta {
     }
 
     #[doc(alias = "gst_custom_meta_get_structure")]
+    #[inline]
     pub fn mut_structure(&mut self) -> &mut crate::StructureRef {
         unsafe {
             crate::StructureRef::from_glib_borrow_mut(ffi::gst_custom_meta_get_structure(
@@ -678,6 +721,7 @@ impl CustomMeta {
     }
 
     #[doc(alias = "gst_custom_meta_has_name")]
+    #[inline]
     pub fn has_name(&self, name: &str) -> bool {
         unsafe {
             from_glib(ffi::gst_custom_meta_has_name(
