@@ -1,19 +1,23 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::borrow::{Borrow, BorrowMut, ToOwned};
-use std::ffi::CStr;
-use std::fmt;
-use std::mem;
-use std::ops::{Deref, DerefMut};
-use std::ptr;
-use std::str;
+use std::{
+    borrow::{Borrow, BorrowMut, ToOwned},
+    fmt,
+    marker::PhantomData,
+    mem,
+    ops::{Deref, DerefMut},
+    ptr, str,
+};
+
+use glib::{
+    prelude::*,
+    translate::*,
+    value::{FromValue, SendValue, ToSendValue},
+    IntoGStr,
+};
 
 use crate::Fraction;
 
-use glib::translate::*;
-use glib::value::{FromValue, SendValue, ToSendValue};
-use glib::StaticType;
-
 #[derive(Clone, Debug, Eq, PartialEq, thiserror::Error)]
 pub enum GetError<E: std::error::Error> {
     #[error("GetError: Structure field with name {name} not found")]
@@ -46,24 +50,25 @@ unsafe impl Sync for Structure {}
 
 impl Structure {
     #[doc(alias = "gst_structure_new")]
-    pub fn builder(name: &str) -> Builder {
-        assert_initialized_main_thread!();
+    pub fn builder(name: impl IntoGStr) -> Builder {
+        skip_assert_initialized!();
         Builder::new(name)
     }
 
     #[doc(alias = "gst_structure_new_empty")]
-    pub fn new_empty(name: &str) -> Structure {
+    pub fn new_empty(name: impl IntoGStr) -> Structure {
         assert_initialized_main_thread!();
         unsafe {
-            let ptr = ffi::gst_structure_new_empty(name.to_glib_none().0);
-            assert!(!ptr.is_null());
+            let ptr = name.run_with_gstr(|name| ffi::gst_structure_new_empty(name.as_ptr()));
+            debug_assert!(!ptr.is_null());
             Structure(ptr::NonNull::new_unchecked(ptr))
         }
     }
 
     #[doc(alias = "gst_structure_new")]
-    pub fn new(name: &str, values: &[(&str, &(dyn ToSendValue + Sync))]) -> Structure {
-        assert_initialized_main_thread!();
+    #[deprecated = "Use `Structure::builder()` or `Structure::new_empty()`"]
+    pub fn new(name: impl IntoGStr, values: &[(&str, &(dyn ToSendValue + Sync))]) -> Structure {
+        skip_assert_initialized!();
         let mut structure = Structure::new_empty(name);
 
         for &(f, v) in values {
@@ -74,11 +79,11 @@ impl Structure {
     }
 
     #[allow(clippy::should_implement_trait)]
-    pub fn from_iter<'a>(
-        name: &str,
-        iter: impl IntoIterator<Item = (&'a str, SendValue)>,
+    pub fn from_iter(
+        name: impl IntoGStr,
+        iter: impl IntoIterator<Item = (impl IntoGStr, SendValue)>,
     ) -> Structure {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let mut structure = Structure::new_empty(name);
 
         iter.into_iter()
@@ -89,6 +94,7 @@ impl Structure {
 }
 
 impl IntoGlibPtr<*mut ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn into_glib_ptr(self) -> *mut ffi::GstStructure {
         let s = mem::ManuallyDrop::new(self);
         s.0.as_ptr()
@@ -98,40 +104,46 @@ impl IntoGlibPtr<*mut ffi::GstStructure> for Structure {
 impl Deref for Structure {
     type Target = StructureRef;
 
+    #[inline]
     fn deref(&self) -> &StructureRef {
         unsafe { &*(self.0.as_ptr() as *const StructureRef) }
     }
 }
 
 impl DerefMut for Structure {
+    #[inline]
     fn deref_mut(&mut self) -> &mut StructureRef {
         unsafe { &mut *(self.0.as_ptr() as *mut StructureRef) }
     }
 }
 
 impl AsRef<StructureRef> for Structure {
+    #[inline]
     fn as_ref(&self) -> &StructureRef {
         self.deref()
     }
 }
 
 impl AsMut<StructureRef> for Structure {
+    #[inline]
     fn as_mut(&mut self) -> &mut StructureRef {
         self.deref_mut()
     }
 }
 
 impl Clone for Structure {
+    #[inline]
     fn clone(&self) -> Self {
         unsafe {
             let ptr = ffi::gst_structure_copy(self.0.as_ref());
-            assert!(!ptr.is_null());
+            debug_assert!(!ptr.is_null());
             Structure(ptr::NonNull::new_unchecked(ptr))
         }
     }
 }
 
 impl Drop for Structure {
+    #[inline]
     fn drop(&mut self) {
         unsafe { ffi::gst_structure_free(self.0.as_mut()) }
     }
@@ -178,7 +190,8 @@ impl str::FromStr for Structure {
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         assert_initialized_main_thread!();
         unsafe {
-            let structure = ffi::gst_structure_from_string(s.to_glib_none().0, ptr::null_mut());
+            let structure =
+                s.run_with_gstr(|s| ffi::gst_structure_from_string(s.as_ptr(), ptr::null_mut()));
             if structure.is_null() {
                 Err(glib::bool_error!("Failed to parse structure from string"))
             } else {
@@ -189,12 +202,14 @@ impl str::FromStr for Structure {
 }
 
 impl Borrow<StructureRef> for Structure {
+    #[inline]
     fn borrow(&self) -> &StructureRef {
         self.as_ref()
     }
 }
 
 impl BorrowMut<StructureRef> for Structure {
+    #[inline]
     fn borrow_mut(&mut self) -> &mut StructureRef {
         self.as_mut()
     }
@@ -206,94 +221,106 @@ impl ToOwned for StructureRef {
     fn to_owned(&self) -> Structure {
         unsafe {
             let ptr = ffi::gst_structure_copy(&self.0);
-            assert!(!ptr.is_null());
+            debug_assert!(!ptr.is_null());
             Structure(ptr::NonNull::new_unchecked(ptr))
         }
     }
 }
 
 impl glib::types::StaticType for Structure {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_structure_get_type()) }
     }
 }
 
 impl<'a> ToGlibPtr<'a, *const ffi::GstStructure> for Structure {
-    type Storage = &'a Self;
+    type Storage = PhantomData<&'a Self>;
 
+    #[inline]
     fn to_glib_none(&'a self) -> Stash<'a, *const ffi::GstStructure, Self> {
-        unsafe { Stash(self.0.as_ref(), self) }
+        unsafe { Stash(self.0.as_ref(), PhantomData) }
     }
 
+    #[inline]
     fn to_glib_full(&self) -> *const ffi::GstStructure {
         unsafe { ffi::gst_structure_copy(self.0.as_ref()) }
     }
 }
 
 impl<'a> ToGlibPtr<'a, *mut ffi::GstStructure> for Structure {
-    type Storage = &'a Self;
+    type Storage = PhantomData<&'a Self>;
 
+    #[inline]
     fn to_glib_none(&'a self) -> Stash<'a, *mut ffi::GstStructure, Self> {
         unsafe {
             Stash(
                 self.0.as_ref() as *const ffi::GstStructure as *mut ffi::GstStructure,
-                self,
+                PhantomData,
             )
         }
     }
 
+    #[inline]
     fn to_glib_full(&self) -> *mut ffi::GstStructure {
         unsafe { ffi::gst_structure_copy(self.0.as_ref()) }
     }
 }
 
 impl<'a> ToGlibPtrMut<'a, *mut ffi::GstStructure> for Structure {
-    type Storage = &'a mut Self;
+    type Storage = PhantomData<&'a mut Self>;
 
+    #[inline]
     fn to_glib_none_mut(&'a mut self) -> StashMut<*mut ffi::GstStructure, Self> {
-        unsafe { StashMut(self.0.as_mut(), self) }
+        unsafe { StashMut(self.0.as_mut(), PhantomData) }
     }
 }
 
 impl FromGlibPtrNone<*const ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_none(ptr: *const ffi::GstStructure) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         let ptr = ffi::gst_structure_copy(ptr);
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         Structure(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 impl FromGlibPtrNone<*mut ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_none(ptr: *mut ffi::GstStructure) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         let ptr = ffi::gst_structure_copy(ptr);
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         Structure(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 impl FromGlibPtrFull<*const ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_full(ptr: *const ffi::GstStructure) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         Structure(ptr::NonNull::new_unchecked(ptr as *mut ffi::GstStructure))
     }
 }
 
 impl FromGlibPtrFull<*mut ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_full(ptr: *mut ffi::GstStructure) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         Structure(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 impl FromGlibPtrBorrow<*const ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_borrow(ptr: *const ffi::GstStructure) -> Borrowed<Self> {
         Borrowed::new(from_glib_full(ptr))
     }
 }
 
 impl FromGlibPtrBorrow<*mut ffi::GstStructure> for Structure {
+    #[inline]
     unsafe fn from_glib_borrow(ptr: *mut ffi::GstStructure) -> Borrowed<Self> {
         Borrowed::new(from_glib_full(ptr))
     }
@@ -349,10 +376,26 @@ impl glib::value::ToValueOptional for Structure {
     }
 }
 
+impl From<Structure> for glib::Value {
+    fn from(v: Structure) -> glib::Value {
+        skip_assert_initialized!();
+        let mut value = glib::Value::for_value_type::<Structure>();
+        unsafe {
+            glib::gobject_ffi::g_value_take_boxed(
+                value.to_glib_none_mut().0,
+                glib::translate::IntoGlibPtr::<*mut ffi::GstStructure>::into_glib_ptr(v) as *mut _,
+            )
+        }
+        value
+    }
+}
+
 impl GlibPtrDefault for Structure {
     type GlibType = *mut ffi::GstStructure;
 }
 
+unsafe impl TransparentPtrType for Structure {}
+
 #[repr(transparent)]
 #[doc(alias = "GstStructure")]
 pub struct StructureRef(ffi::GstStructure);
@@ -361,22 +404,26 @@ unsafe impl Send for StructureRef {}
 unsafe impl Sync for StructureRef {}
 
 impl StructureRef {
+    #[inline]
     pub unsafe fn from_glib_borrow<'a>(ptr: *const ffi::GstStructure) -> &'a StructureRef {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         &*(ptr as *mut StructureRef)
     }
 
+    #[inline]
     pub unsafe fn from_glib_borrow_mut<'a>(ptr: *mut ffi::GstStructure) -> &'a mut StructureRef {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         &mut *(ptr as *mut StructureRef)
     }
 
+    #[inline]
     pub fn as_ptr(&self) -> *const ffi::GstStructure {
         self as *const Self as *const ffi::GstStructure
     }
 
+    #[inline]
     pub fn as_mut_ptr(&self) -> *mut ffi::GstStructure {
         self as *const Self as *mut ffi::GstStructure
     }
@@ -384,7 +431,7 @@ impl StructureRef {
     #[doc(alias = "gst_structure_get")]
     pub fn get<'a, T: FromValue<'a>>(
         &'a self,
-        name: &str,
+        name: impl IntoGStr,
     ) -> Result<T, GetError<<<T as FromValue<'a>>::Checker as glib::value::ValueTypeChecker>::Error>>
     {
         let name = glib::Quark::from_str(name);
@@ -394,7 +441,7 @@ impl StructureRef {
     #[doc(alias = "gst_structure_get")]
     pub fn get_optional<'a, T: FromValue<'a>>(
         &'a self,
-        name: &str,
+        name: impl IntoGStr,
     ) -> Result<
         Option<T>,
         GetError<<<T as FromValue<'a>>::Checker as glib::value::ValueTypeChecker>::Error>,
@@ -405,7 +452,10 @@ impl StructureRef {
 
     #[doc(alias = "get_value")]
     #[doc(alias = "gst_structure_get_value")]
-    pub fn value(&self, name: &str) -> Result<&SendValue, GetError<std::convert::Infallible>> {
+    pub fn value(
+        &self,
+        name: impl IntoGStr,
+    ) -> Result<&SendValue, GetError<std::convert::Infallible>> {
         let name = glib::Quark::from_str(name);
         self.value_by_quark(name)
     }
@@ -457,25 +507,23 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_set")]
-    pub fn set<T: ToSendValue + Sync>(&mut self, name: &str, value: T) {
-        let value = value.to_send_value();
+    pub fn set(&mut self, name: impl IntoGStr, value: impl Into<glib::Value> + Send) {
+        let value = glib::SendValue::from_owned(value);
         self.set_value(name, value);
     }
 
     #[doc(alias = "gst_structure_set_value")]
-    pub fn set_value(&mut self, name: &str, value: SendValue) {
+    pub fn set_value(&mut self, name: impl IntoGStr, value: SendValue) {
         unsafe {
-            ffi::gst_structure_take_value(
-                &mut self.0,
-                name.to_glib_none().0,
-                &mut value.into_raw(),
-            );
+            name.run_with_gstr(|name| {
+                ffi::gst_structure_take_value(&mut self.0, name.as_ptr(), &mut value.into_raw())
+            });
         }
     }
 
     #[doc(alias = "gst_structure_id_set")]
-    pub fn set_by_quark<T: ToSendValue + Sync>(&mut self, name: glib::Quark, value: T) {
-        let value = value.to_send_value();
+    pub fn set_by_quark(&mut self, name: glib::Quark, value: impl Into<glib::Value> + Send) {
+        let value = glib::SendValue::from_owned(value);
         self.set_value_by_quark(name, value);
     }
 
@@ -488,12 +536,8 @@ impl StructureRef {
 
     #[doc(alias = "get_name")]
     #[doc(alias = "gst_structure_get_name")]
-    pub fn name<'a>(&self) -> &'a str {
-        unsafe {
-            CStr::from_ptr(ffi::gst_structure_get_name(&self.0))
-                .to_str()
-                .unwrap()
-        }
+    pub fn name<'a>(&self) -> &'a glib::GStr {
+        unsafe { glib::GStr::from_ptr(ffi::gst_structure_get_name(&self.0)) }
     }
 
     #[doc(alias = "gst_structure_get_name_id")]
@@ -502,8 +546,10 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_set_name")]
-    pub fn set_name(&mut self, name: &str) {
-        unsafe { ffi::gst_structure_set_name(&mut self.0, name.to_glib_none().0) }
+    pub fn set_name(&mut self, name: impl IntoGStr) {
+        unsafe {
+            name.run_with_gstr(|name| ffi::gst_structure_set_name(&mut self.0, name.as_ptr()))
+        }
     }
 
     #[doc(alias = "gst_structure_has_name")]
@@ -512,23 +558,24 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_has_field")]
-    pub fn has_field(&self, field: &str) -> bool {
+    pub fn has_field(&self, field: impl IntoGStr) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_has_field(
-                &self.0,
-                field.to_glib_none().0,
-            ))
+            field.run_with_gstr(|field| {
+                from_glib(ffi::gst_structure_has_field(&self.0, field.as_ptr()))
+            })
         }
     }
 
     #[doc(alias = "gst_structure_has_field_typed")]
-    pub fn has_field_with_type(&self, field: &str, type_: glib::Type) -> bool {
+    pub fn has_field_with_type(&self, field: impl IntoGStr, type_: glib::Type) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_has_field_typed(
-                &self.0,
-                field.to_glib_none().0,
-                type_.into_glib(),
-            ))
+            field.run_with_gstr(|field| {
+                from_glib(ffi::gst_structure_has_field_typed(
+                    &self.0,
+                    field.as_ptr(),
+                    type_.into_glib(),
+                ))
+            })
         }
     }
 
@@ -549,15 +596,17 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_remove_field")]
-    pub fn remove_field(&mut self, field: &str) {
+    pub fn remove_field(&mut self, field: impl IntoGStr) {
         unsafe {
-            ffi::gst_structure_remove_field(&mut self.0, field.to_glib_none().0);
+            field.run_with_gstr(|field| {
+                ffi::gst_structure_remove_field(&mut self.0, field.as_ptr())
+            });
         }
     }
 
     #[doc(alias = "gst_structure_remove_fields")]
-    pub fn remove_fields(&mut self, fields: &[&str]) {
-        for f in fields {
+    pub fn remove_fields(&mut self, fields: impl IntoIterator<Item = impl IntoGStr>) {
+        for f in fields.into_iter() {
             self.remove_field(f)
         }
     }
@@ -579,16 +628,16 @@ impl StructureRef {
 
     #[doc(alias = "get_nth_field_name")]
     #[doc(alias = "gst_structure_nth_field_name")]
-    pub fn nth_field_name<'a>(&self, idx: u32) -> Option<&'a str> {
+    pub fn nth_field_name<'a>(&self, idx: u32) -> Option<&'a glib::GStr> {
         if idx >= self.n_fields() {
             return None;
         }
 
         unsafe {
             let field_name = ffi::gst_structure_nth_field_name(&self.0, idx);
-            assert!(!field_name.is_null());
+            debug_assert!(!field_name.is_null());
 
-            Some(CStr::from_ptr(field_name).to_str().unwrap())
+            Some(glib::GStr::from_ptr(field_name))
         }
     }
 
@@ -618,75 +667,86 @@ impl StructureRef {
     }
 
     #[doc(alias = "gst_structure_fixate_field")]
-    pub fn fixate_field(&mut self, name: &str) -> bool {
+    pub fn fixate_field(&mut self, name: impl IntoGStr) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field(
-                &mut self.0,
-                name.to_glib_none().0,
-            ))
+            name.run_with_gstr(|name| {
+                from_glib(ffi::gst_structure_fixate_field(&mut self.0, name.as_ptr()))
+            })
         }
     }
 
     #[doc(alias = "gst_structure_fixate_field_boolean")]
-    pub fn fixate_field_bool(&mut self, name: &str, target: bool) -> bool {
+    pub fn fixate_field_bool(&mut self, name: impl IntoGStr, target: bool) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field_boolean(
-                &mut self.0,
-                name.to_glib_none().0,
-                target.into_glib(),
-            ))
+            name.run_with_gstr(|name| {
+                from_glib(ffi::gst_structure_fixate_field_boolean(
+                    &mut self.0,
+                    name.as_ptr(),
+                    target.into_glib(),
+                ))
+            })
         }
     }
 
     #[doc(alias = "gst_structure_fixate_field_string")]
-    pub fn fixate_field_str(&mut self, name: &str, target: &str) -> bool {
+    pub fn fixate_field_str(&mut self, name: impl IntoGStr, target: impl IntoGStr) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field_string(
-                &mut self.0,
-                name.to_glib_none().0,
-                target.to_glib_none().0,
-            ))
+            name.run_with_gstr(|name| {
+                target.run_with_gstr(|target| {
+                    from_glib(ffi::gst_structure_fixate_field_string(
+                        &mut self.0,
+                        name.as_ptr(),
+                        target.as_ptr(),
+                    ))
+                })
+            })
         }
     }
 
     #[doc(alias = "gst_structure_fixate_field_nearest_double")]
-    pub fn fixate_field_nearest_double(&mut self, name: &str, target: f64) -> bool {
+    pub fn fixate_field_nearest_double(&mut self, name: impl IntoGStr, target: f64) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field_nearest_double(
-                &mut self.0,
-                name.to_glib_none().0,
-                target,
-            ))
+            name.run_with_gstr(|name| {
+                from_glib(ffi::gst_structure_fixate_field_nearest_double(
+                    &mut self.0,
+                    name.as_ptr(),
+                    target,
+                ))
+            })
         }
     }
 
     #[doc(alias = "gst_structure_fixate_field_nearest_fraction")]
-    pub fn fixate_field_nearest_fraction<T: Into<Fraction>>(
+    pub fn fixate_field_nearest_fraction(
         &mut self,
-        name: &str,
-        target: T,
+        name: impl IntoGStr,
+        target: impl Into<Fraction>,
     ) -> bool {
         skip_assert_initialized!();
 
         let target = target.into();
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field_nearest_fraction(
-                &mut self.0,
-                name.to_glib_none().0,
-                target.numer(),
-                target.denom(),
-            ))
+            name.run_with_gstr(|name| {
+                from_glib(ffi::gst_structure_fixate_field_nearest_fraction(
+                    &mut self.0,
+                    name.as_ptr(),
+                    target.numer(),
+                    target.denom(),
+                ))
+            })
         }
     }
 
     #[doc(alias = "gst_structure_fixate_field_nearest_int")]
-    pub fn fixate_field_nearest_int(&mut self, name: &str, target: i32) -> bool {
+    pub fn fixate_field_nearest_int(&mut self, name: impl IntoGStr, target: i32) -> bool {
         unsafe {
-            from_glib(ffi::gst_structure_fixate_field_nearest_int(
-                &mut self.0,
-                name.to_glib_none().0,
-                target,
-            ))
+            name.run_with_gstr(|name| {
+                from_glib(ffi::gst_structure_fixate_field_nearest_int(
+                    &mut self.0,
+                    name.as_ptr(),
+                    target,
+                ))
+            })
         }
     }
 
@@ -829,6 +889,7 @@ impl PartialEq for StructureRef {
 impl Eq for StructureRef {}
 
 impl glib::types::StaticType for StructureRef {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_structure_get_type()) }
     }
@@ -895,7 +956,7 @@ impl<'a> FieldIterator<'a> {
 }
 
 impl<'a> Iterator for FieldIterator<'a> {
-    type Item = &'static str;
+    type Item = &'static glib::GStr;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.idx >= self.n_fields {
@@ -945,7 +1006,7 @@ impl<'a> Iter<'a> {
 }
 
 impl<'a> Iterator for Iter<'a> {
-    type Item = (&'static str, &'a SendValue);
+    type Item = (&'static glib::GStr, &'a SendValue);
 
     fn next(&mut self) -> Option<Self::Item> {
         let f = self.iter.next()?;
@@ -995,7 +1056,7 @@ impl<'a> std::iter::FusedIterator for Iter<'a> {}
 
 impl<'a> IntoIterator for &'a StructureRef {
     type IntoIter = Iter<'a>;
-    type Item = (&'static str, &'a SendValue);
+    type Item = (&'static glib::GStr, &'a SendValue);
 
     fn into_iter(self) -> Self::IntoIter {
         self.iter()
@@ -1008,12 +1069,24 @@ impl<'a> std::iter::Extend<(&'a str, SendValue)> for StructureRef {
     }
 }
 
+impl<'a> std::iter::Extend<(&'a glib::GStr, SendValue)> for StructureRef {
+    fn extend<T: IntoIterator<Item = (&'a glib::GStr, SendValue)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|(f, v)| self.set_value(f, v));
+    }
+}
+
 impl std::iter::Extend<(String, SendValue)> for StructureRef {
     fn extend<T: IntoIterator<Item = (String, SendValue)>>(&mut self, iter: T) {
         iter.into_iter().for_each(|(f, v)| self.set_value(&f, v));
     }
 }
 
+impl std::iter::Extend<(glib::GString, SendValue)> for StructureRef {
+    fn extend<T: IntoIterator<Item = (glib::GString, SendValue)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|(f, v)| self.set_value(&f, v));
+    }
+}
+
 impl std::iter::Extend<(glib::Quark, SendValue)> for StructureRef {
     fn extend<T: IntoIterator<Item = (glib::Quark, SendValue)>>(&mut self, iter: T) {
         iter.into_iter()
@@ -1028,14 +1101,14 @@ pub struct Builder {
 }
 
 impl Builder {
-    fn new(name: &str) -> Self {
+    fn new(name: impl IntoGStr) -> Self {
         skip_assert_initialized!();
         Builder {
             s: Structure::new_empty(name),
         }
     }
 
-    pub fn field<V: ToSendValue + Sync>(mut self, name: &str, value: V) -> Self {
+    pub fn field(mut self, name: impl IntoGStr, value: impl Into<glib::Value> + Send) -> Self {
         self.s.set(name, value);
         self
     }
@@ -1051,6 +1124,7 @@ mod tests {
     use super::*;
 
     #[test]
+    #[allow(deprecated)]
     fn new_set_get() {
         use glib::{value, Type};
 
@@ -1112,7 +1186,7 @@ mod tests {
 
         let s = Structure::builder("test")
             .field("f1", "abc")
-            .field("f2", &String::from("bcd"))
+            .field("f2", String::from("bcd"))
             .field("f3", 123i32)
             .build();
 
@@ -1153,7 +1227,7 @@ mod tests {
 
         let s = Structure::builder("test")
             .field("f1", "abc")
-            .field("f2", &String::from("bcd"))
+            .field("f2", String::from("bcd"))
             .field("f3", 123i32)
             .build();
 
@@ -1186,6 +1260,6 @@ mod tests {
             .field("f6", crate::List::new(["d", "e", "f"]))
             .build();
 
-        assert_eq!(format!("{:?}", s), "Structure(test { f1: (gchararray) \"abc\", f2: (gchararray) \"bcd\", f3: (gint) 123, f4: Structure(nested { badger: (gboolean) TRUE }), f5: Array([(gchararray) \"a\", (gchararray) \"b\", (gchararray) \"c\"]), f6: List([(gchararray) \"d\", (gchararray) \"e\", (gchararray) \"f\"]) })");
+        assert_eq!(format!("{s:?}"), "Structure(test { f1: (gchararray) \"abc\", f2: (gchararray) \"bcd\", f3: (gint) 123, f4: Structure(nested { badger: (gboolean) TRUE }), f5: Array([(gchararray) \"a\", (gchararray) \"b\", (gchararray) \"c\"]), f6: List([(gchararray) \"d\", (gchararray) \"e\", (gchararray) \"f\"]) })");
     }
 }
