@@ -1,21 +1,14 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+use std::{borrow::Cow, sync::atomic};
+
+use glib::{subclass::prelude::*, translate::*};
+
 use super::prelude::*;
-use crate::prelude::*;
-use glib::subclass::prelude::*;
-use glib::translate::*;
-
-use crate::Element;
-use crate::Event;
-use crate::PadTemplate;
-use crate::QueryRef;
-use crate::StateChange;
-use crate::StateChangeError;
-use crate::StateChangeReturn;
-use crate::StateChangeSuccess;
-
-use std::borrow::Cow;
-use std::sync::atomic;
+use crate::{
+    prelude::*, Element, Event, PadTemplate, QueryRef, StateChange, StateChangeError,
+    StateChangeReturn, StateChangeSuccess,
+};
 
 #[derive(Debug, Clone)]
 pub struct ElementMetadata {
@@ -157,12 +150,7 @@ pub trait ElementImplExt: ObjectSubclass {
 
     fn panicked(&self) -> &atomic::AtomicBool;
 
-    fn catch_panic<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R, P: IsA<Element>>(
-        &self,
-        element: &P,
-        fallback: G,
-        f: F,
-    ) -> R;
+    fn catch_panic<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R>(&self, fallback: G, f: F) -> R;
 
     fn catch_panic_pad_function<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R>(
         parent: Option<&crate::Object>,
@@ -331,25 +319,14 @@ impl<T: ElementImpl> ElementImplExt for T {
         }
     }
 
+    #[inline(never)]
     fn panicked(&self) -> &atomic::AtomicBool {
         self.instance_data::<atomic::AtomicBool>(crate::Element::static_type())
             .expect("instance not initialized correctly")
     }
 
-    fn catch_panic<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R, P: IsA<Element>>(
-        &self,
-        element: &P,
-        fallback: G,
-        f: F,
-    ) -> R {
-        unsafe {
-            assert!(element.type_().is_a(T::type_()));
-            let ptr: *mut ffi::GstElement = element.as_ptr() as *mut _;
-            let instance = &*(ptr as *mut T::Instance);
-            let imp = instance.imp();
-
-            panic_to_error!(imp, fallback(), { f(imp) })
-        }
+    fn catch_panic<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R>(&self, fallback: G, f: F) -> R {
+        panic_to_error!(self, fallback(), { f(self) })
     }
 
     fn catch_panic_pad_function<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R>(
@@ -357,15 +334,10 @@ impl<T: ElementImpl> ElementImplExt for T {
         fallback: G,
         f: F,
     ) -> R {
-        unsafe {
-            let wrap = parent.as_ref().unwrap().downcast_ref::<Element>().unwrap();
-            assert!(wrap.type_().is_a(Self::type_()));
-            let ptr: *mut ffi::GstElement = wrap.to_glib_none().0;
-            let instance = &*(ptr as *mut Self::Instance);
-            let imp = instance.imp();
+        let element = parent.unwrap().dynamic_cast_ref::<Self::Type>().unwrap();
+        let imp = element.imp();
 
-            panic_to_error!(imp, fallback(), { f(imp) })
-        }
+        panic_to_error!(imp, fallback(), { f(imp) })
     }
 
     fn post_error_message(&self, msg: crate::ErrorMessage) {
@@ -545,7 +517,7 @@ unsafe extern "C" fn element_provide_clock<T: ElementImpl>(
     let instance = &*(ptr as *mut T::Instance);
     let imp = instance.imp();
 
-    panic_to_error!(imp, None, { imp.provide_clock() }).to_glib_full()
+    panic_to_error!(imp, None, { imp.provide_clock() }).into_glib_ptr()
 }
 
 unsafe extern "C" fn element_post_message<T: ElementImpl>(
@@ -562,9 +534,9 @@ unsafe extern "C" fn element_post_message<T: ElementImpl>(
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use std::sync::atomic;
 
+    use super::*;
     use crate::ElementFactory;
 
     pub mod imp {
@@ -736,7 +708,7 @@ mod tests {
 
     impl TestElement {
         pub fn new(name: Option<&str>) -> Self {
-            glib::Object::new(&[("name", &name)])
+            glib::Object::builder().property("name", name).build()
         }
     }
 
@@ -749,7 +721,7 @@ mod tests {
         assert_eq!(element.name(), "test");
 
         assert_eq!(
-            element.metadata(&crate::ELEMENT_METADATA_LONGNAME),
+            element.metadata(crate::ELEMENT_METADATA_LONGNAME),
             Some("Test Element")
         );
 
